const TCHAR g_pszSvnVersion[] = wxT("esportato");
const TCHAR g_pszAppVersion[] = wxT("0.7");
#define CAMELOT_VERSION_MAJOR 
#define CAMELOT_VERSION_MINOR 
#define CAMELOT_VERSION 0.7
#define CAMELOT_VERSION_STRING wxT("0.7 (loris)")
#define CAMELOT_BUILD_DATE _T("25-Dec-06 15:16")
